package dbridge;

import java.awt.*;

import javax.swing.*;

import java.util.*;
import java.awt.event.*;
import javax.swing.BorderFactory;
import javax.swing.border.CompoundBorder;

/**
 * <p>Title: dsUserSettings </p>
 * <p>Description: Main User Interface Form</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: aItLab</p>
 * @author Dmitry Stepankov
 * @version 1.0
 */



/**
 * The Interface for all User GUI objects
 */
interface dsGUIInterface {
 
	/**
	 * set the GUI object
	 * @param obj
	 */
	public void setObject(Object obj);

	/**
	 * save the State of current GUI object
	 *
	 */
	public void saveState();

	/**
	 * check the Object for errors
	 * @return
	 */
	public boolean check();

}


/**
 * The Base Abstract class for all of the GUI objects
 * @author bob
 */
abstract class dsUserGUI
    extends JPanel
    implements dsGUIInterface {
	
  /**
   * The main Constructor
   * @param controller
   */
  protected dsUserGUI(UserController controller) {
    mController = controller;
  }

  /**
   * Notify the up-level object, that we changed the User DB
   *
   */
  public void UserChanged() {
    mController.getChangesTracker().UsersChanged = true;
  }

  /**
   * The up-level object
   */
  protected UserController mController = null;

  /**
   * The methods inherited from dsGUIInterface
   */
  abstract public void setObject(Object obj);

  abstract public void saveState();

  abstract public boolean check();

}


/**
 * The Adapter and Listener which tracks if user has changed something
 * @author bob
 *
 */
class dsUserGUI_keyAdapter
    extends java.awt.event.KeyAdapter implements ActionListener{
  
 /**
  * The GUI object to listen
  */
  private dsUserGUI adaptee;

  dsUserGUI_keyAdapter(dsUserGUI obj) {
    adaptee = obj;
  }

  /**
   * track the key typing events
   */
  public void keyTyped(KeyEvent e) {
    adaptee.UserChanged();
  }
  
  /**
   * track the other events
   */
  public void actionPerformed(ActionEvent e) {
	  adaptee.UserChanged();
    }

}

/**
 * BASE Class for Server Table Panels
 * @author bob
 */
abstract class dsServersTablePanel
    extends JPanel
    implements dsGUIInterface {

  public static String ADD_COMMAND = "add";
  public static String REMOVE_COMMAND = "remove";
  public static String EDIT_COMMAND = "edit";
  public static String CHECK_COMMAND = "check";

  protected JLabel lTitle;
  protected ResourceBundle lbls;
  protected boolean mInitialized = false;
  protected dsServersTable mainTable;
  protected UserController mController = null;
  protected dbUser mUser;
  protected IUpdateUser mUpdUser = null;

  protected JScrollPane mainPane = null;
  private JButton bAdd;
  private JButton bEdit;
  private JButton bRemove;
  private JButton bCheck;

  private ActionListener mListener = null;

  protected dsServersTablePanel(UserController controller) {
    mInitialized = false;
    mController = controller;
  }

  protected dsServersTablePanel(IUpdateUser controller) {
    mInitialized = false;
    mUpdUser = controller;
  }

  protected void disableCheckButton() {
    this.bCheck.setVisible(false);
  }

  public void setActionListener(ActionListener listener) {
    mListener = listener;
  }

  public UserController getUserController() {
    return mController;
  }

  public void UserUpdated() {
    if (mUpdUser != null) {
      mUpdUser.userChanged();
    }
    if (mController != null) {
      mController.getChangesTracker().UsersChanged = true;
    }
  }

  public Object getSelectedObj() {

    Object res = mainTable.getSelection();
    if (res == null) {
      JOptionPane.showMessageDialog(this, lbls.getString("noSelErr"),
                                    lbls.getString("error"), 1);
    }
    return res;
  }

  public dbUser getUser() {
    return mUser;
  }

  public void Update() {
    if (mInitialized) {
      mainTable.updateUI();
    }
  }

  protected void Init() {

    Locale currentLocale = BridgeApp.getCurrentLocale();

    lbls = ResourceBundle.getBundle(
        "dbridge.prop.dsUserSettingsBundle",
        currentLocale);
    BorderLayout main_layout = new BorderLayout();

    this.setLayout(main_layout);

    Box mainBox = Box.createVerticalBox();
    lTitle = new JLabel();
    lTitle.setFont(new java.awt.Font("Dialog", 1, 14));
    JPanel titlePan = new JPanel();
    titlePan.setMaximumSize(new Dimension(10000, 40));
    titlePan.setBorder(BorderFactory.createLoweredBevelBorder());
    titlePan.add(lTitle);
    mainBox.add(titlePan);

    mainPane = new JScrollPane();
    mainPane.setPreferredSize(new Dimension(150, 200));
    mainBox.add(mainPane);

    bAdd = new JButton();
    bAdd.setText(lbls.getString("bAdd"));
    bAdd.setActionCommand(ADD_COMMAND);
    bAdd.addActionListener(mListener);
    bEdit = new JButton();
    bEdit.setText(lbls.getString("bEdit"));
    bEdit.setActionCommand(EDIT_COMMAND);
    bEdit.addActionListener(mListener);
    bRemove = new JButton();
    bRemove.setText(lbls.getString("bRemove"));
    bRemove.setActionCommand(REMOVE_COMMAND);
    bRemove.addActionListener(mListener);

    bCheck = new JButton();
    bCheck.setText(lbls.getString("bCheck"));
    bCheck.setActionCommand(CHECK_COMMAND);
    bCheck.addActionListener(mListener);

    JPanel butPanel = new JPanel();
    butPanel.setMaximumSize(new Dimension(10000, 40));
    butPanel.setBorder(BorderFactory.createEtchedBorder());
    butPanel.add(bAdd);
    butPanel.add(bEdit);
    butPanel.add(bRemove);
    butPanel.add(bCheck);

    mainBox.add(butPanel);
    this.add(mainBox, BorderLayout.CENTER);
    this.add(Box.createVerticalStrut(10), BorderLayout.NORTH);
    this.add(Box.createVerticalStrut(10), BorderLayout.SOUTH);
    this.add(Box.createHorizontalStrut(10), BorderLayout.EAST);
    this.add(Box.createHorizontalStrut(10), BorderLayout.WEST);

  }

  abstract public void setObject(Object data);

  abstract public void saveState();

  abstract public boolean check();

}

/**
 * The Main Panel for Manual user Settings
 * @author bob
 *
 */
class dsUserSettings
    extends dsUserGUI {

  private dbUser mUser;
  private ResourceBundle lbls;
  private JLabel jlLogin;
  private JTextField jtfName;
  private JPasswordField jtfPassword;
  private JPasswordField jtfConfirm;
  private dsKeyIDTable key_table;

  private boolean mInitialized = false;
  private UserController mController;

  public dsUserSettings(UserController controller) {
    super(controller);
    mInitialized = false;
    mController = controller;
  }

  public void setObject(Object obj) throws HeadlessException {
    mUser = (dbUser) ( (userGUIObject) obj).getObject();
    if (!mInitialized) {
      try {
        jbInit();
      }
      catch (Exception e) {
        e.printStackTrace();
      }
      mInitialized = true;

    }
    else {
      this.fill();
      
    }
  }

  public void saveState() {
    mUser.setName(jtfName.getText());
    mUser.setPassword(new String(jtfPassword.getPassword()));
    //mUser.setDecryptPwd(jtfPassword_decr.getText());
  }

  public boolean check() {
    boolean res = false;
    String pass1 = new String(jtfPassword.getPassword());
    String pass_conf = new String(jtfConfirm.getPassword());
    
    if (!pass1.equals(pass_conf)) {
      JOptionPane.showMessageDialog(null,
                                    lbls.getString("invalid_pass"),
                                    lbls.getString("error"),
                                    JOptionPane.ERROR_MESSAGE);
      res = false;
    }
    return res;
  }

  private void jbInit() throws Exception {

    Locale currentLocale = BridgeApp.getCurrentLocale();
    lbls = ResourceBundle.getBundle(
        "dbridge.prop.dsUserSettingsBundle",
        currentLocale);

    dsUserGUI_keyAdapter adapter = new dsUserGUI_keyAdapter(this);

    jlLogin = new JLabel();
    jlLogin.setFont(new java.awt.Font("Dialog", 1, 14));
    jtfName = new JTextField();
    jtfName.addKeyListener(adapter);
    jtfPassword = new JPasswordField();
    jtfPassword.addKeyListener(adapter);
    jtfConfirm = new JPasswordField();
 

    Box mBox1;
    CompoundBorder border1;
    JPanel pane1 = new JPanel();

    JPanel key_pan = new JPanel();
    JPanel pane3 = new JPanel();
    JLabel jlUserName = new JLabel();
    CompoundBorder border2;
    JLabel jlPasswd = new JLabel();
    JLabel jlConfirm = new JLabel();
    JLabel jlPassword_decr = new JLabel();
    JLabel jlConfirm_decr = new JLabel();

    //jlLogin.setText(lbls.getString("login"));
    jlUserName.setText(lbls.getString("name"));
    jlPasswd.setText(lbls.getString("passwd"));
    jlConfirm.setText(lbls.getString("confirm"));
    jlPassword_decr.setText(lbls.getString("passwd"));
    jlConfirm_decr.setText(lbls.getString("confirm"));
    border1 = BorderFactory.createCompoundBorder(BorderFactory.
                                                 createTitledBorder(lbls.
        getString("login_info")), BorderFactory.createEmptyBorder(4, 4, 4, 4));
    border2 = BorderFactory.createCompoundBorder(BorderFactory.
                                                 createTitledBorder(lbls.
        getString("sec_key_pwd")), BorderFactory.createEmptyBorder(4, 4, 4, 4));

    

    pane1.setBorder(border1);
    pane1.setLayout(new GridLayout(0, 2, 20, 2));
    pane1.add(jlUserName);
    pane1.add(jtfName);
    pane1.add(jlPasswd);
    pane1.add(jtfPassword);
    pane1.add(jlConfirm);
    pane1.add(jtfConfirm);
    pane1.setMaximumSize(new Dimension(10000, 60));

    key_table = new dsKeyIDTable(mUser.getGpgKeys(), mController.getChangesTracker());
    
    key_pan.setBorder(border2);
    key_pan.setLayout(new BorderLayout());
    dsAlignments.CreateBorder(key_pan, 10);
    key_pan.add(key_table, BorderLayout.CENTER);
    key_pan.setMaximumSize(new Dimension(10000, 60));

    pane3.setBorder(BorderFactory.createLoweredBevelBorder());
    pane3.add(jlLogin);
    pane3.setMaximumSize(new Dimension(10000, 60));

    mBox1 = Box.createVerticalBox();
    mBox1.add(pane3);
    mBox1.add(pane1);
    mBox1.add(key_pan);
    mBox1.add(Box.createGlue());

    BorderLayout layout = new BorderLayout();
    this.setLayout(layout);

    this.add(mBox1, BorderLayout.CENTER);
    this.add(Box.createVerticalStrut(10), BorderLayout.NORTH);
    this.add(Box.createVerticalStrut(10), BorderLayout.SOUTH);
    this.add(Box.createHorizontalStrut(10), BorderLayout.EAST);
    this.add(Box.createHorizontalStrut(10), BorderLayout.WEST);
    
    this.fill();

  }

  private void fill() {
    jlLogin.setText(mUser.getLogin());
    jtfName.setText(mUser.getName());
    jtfPassword.setText(mUser.getPassword());
    jtfConfirm.setText(mUser.getPassword());
    key_table.setDataVector(mUser.getGpgKeys());

  }

}

/**
 * The Main Panel for AutoUser settings
 * @author bob
 *
 */
class dsAutoUserSettings
    extends dsUserGUI {

  private boolean mInitialized = false;
  private dbUser mUser = null;
  private UserContainer mContainer = null;
  private ResourceBundle lbls;
  private JLabel jlLogin;
  private JTextField jAddrField1;
  private JTextField jtfTimeout;
  private JTextField jKeyField;
  private JTextField jCommentsField;
  
  private JComboBox jMethodDN;
  private JComboBox jMethodSign;
  
  private JLabel jlManUser;
  private JLabel jlServer;
  private JButton jbSelManUser = new JButton();
  private JButton jPKeyBttn = new JButton();

  private boolean mManSel = false;

  public dsAutoUserSettings(UserController controller) {
    super(controller);
    mInitialized = false;
  }

  private void Init() throws Exception {
    Locale currentLocale = BridgeApp.getCurrentLocale();
    lbls = ResourceBundle.getBundle(
        "dbridge.prop.dsUserSettingsBundle",
        currentLocale);

    dsUserGUI_keyAdapter adapter = new dsUserGUI_keyAdapter(this);
    
 
    jlLogin = new JLabel();
    jlLogin.setFont(new java.awt.Font("Dialog", 1, 14));
    jAddrField1 = new JTextField();
    jAddrField1.addKeyListener(adapter);
    jtfTimeout = new JTextField();
    jtfTimeout.addKeyListener(adapter);
    jKeyField = new JTextField();
    jKeyField.addKeyListener(adapter);
    jCommentsField = new JTextField();
    jCommentsField.addKeyListener(adapter);
    
    jMethodDN = new JComboBox();
    jMethodDN.addItem(lbls.getString("mdn_none"));
    jMethodDN.addItem(lbls.getString("mdn_rfc"));
    jMethodDN.addItem(lbls.getString("mdn_pgp"));
    jMethodDN.addItem(lbls.getString("mdn_mix"));
    jMethodDN.addActionListener(adapter);
    
    jMethodSign = new JComboBox();
    jMethodSign.addItem(lbls.getString("ms_none"));
    jMethodSign.addItem(lbls.getString("ms_rfc"));
    jMethodSign.addItem(lbls.getString("ms_pgp"));
    jMethodSign.addItem(lbls.getString("ms_mix"));
    jMethodSign.addActionListener(adapter);

    jlManUser = new JLabel();
    jlManUser.setBorder(BorderFactory.createLoweredBevelBorder());
    jlServer = new JLabel();
    jlServer.setBorder(BorderFactory.createLoweredBevelBorder());
    jbSelManUser = new JButton();

    jbSelManUser.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        jbSelManUser_actionPerformed(e);
      }
    });

    jPKeyBttn = new JButton();
    jPKeyBttn.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        jPKeyBttn_actionPerformed(e);
      }
    });

    jbSelManUser.setText(lbls.getString("select"));
    jPKeyBttn.setText(lbls.getString("public_key"));

    fill();

    JPanel pan0 = new JPanel();
    pan0.setMaximumSize(new Dimension(10000, 60));
    pan0.setBorder(BorderFactory.createLoweredBevelBorder());
    pan0.add(jlLogin);
    Box box1 = Box.createVerticalBox();
    box1.add(pan0);

    JPanel pan1 = new JPanel();
    pan1.setMaximumSize(new Dimension(10000, 60));
    pan1.setBorder(BorderFactory.createCompoundBorder(
        BorderFactory.createTitledBorder(lbls.getString("master")),
        BorderFactory.createEmptyBorder(4, 4, 4, 4)));
    pan1.setLayout(new GridLayout(0, 2, 20, 2));
    pan1.add(jbSelManUser);
    pan1.add(Box.createHorizontalStrut(10));
    pan1.add(new JLabel(lbls.getString("name")));
    pan1.add(jlManUser);
    pan1.add(new JLabel(lbls.getString("smtp")));
    pan1.add(jlServer);
    box1.add(pan1);

    JPanel pan2 = new JPanel();
    pan2.setMaximumSize(new Dimension(10000, 60));
    pan2.setLayout(new GridLayout(0, 2, 20, 2));
    pan2.setBorder(BorderFactory.createCompoundBorder(
        BorderFactory.createTitledBorder(lbls.getString("options")),
        BorderFactory.createEmptyBorder(4, 4, 4, 4)));
    pan2.add(new JLabel(lbls.getString("address")));
    pan2.add(jAddrField1);
    pan2.add(new JLabel(lbls.getString("receiver")));
    pan2.add(jCommentsField);
    pan2.add(jPKeyBttn);
    pan2.add(jKeyField);
    pan2.add(new JLabel(lbls.getString("methodSign")));
    pan2.add(jMethodSign);
    pan2.add(new JLabel(lbls.getString("methodDN")));
    pan2.add(jMethodDN);
    pan2.add(new JLabel(lbls.getString("timeout")));
    pan2.add(jtfTimeout);
    box1.add(pan2);

    //this.add(box1);

    box1.add(Box.createGlue());
    BorderLayout layout = new BorderLayout();
    this.setLayout(layout);

    this.add(box1, BorderLayout.CENTER);
    this.add(Box.createVerticalStrut(10), BorderLayout.NORTH);
    this.add(Box.createVerticalStrut(10), BorderLayout.SOUTH);
    this.add(Box.createHorizontalStrut(10), BorderLayout.EAST);
    this.add(Box.createHorizontalStrut(10), BorderLayout.WEST);

  }

  void jPKeyBttn_actionPerformed(ActionEvent e) {
//   System.out.println("KEY_COMMAND");
    dsGPGKeyDialog gpg = new dsGPGKeyDialog(null, true);
    gpg.setVisible(true);
    if (gpg.isOk()) {
      jKeyField.setText(gpg.getSelectedKey());
      mController.getChangesTracker().UsersChanged = true;
    }
    gpg.dispose();
  }
  
  void jbSelManUser_actionPerformed(ActionEvent e) {

    if (mContainer.size > 0) {
      dsSelUserListDlg dlg = new dsSelUserListDlg(null, true,
                                                  mContainer);
      dlg.setVisible(true);
      if (dlg.isOk()) {
        // copy a Smtp record from a Master to the autouser
        String uname = dlg.getSelectedUser();
        dbSmtpRecord rec = mContainer.getUser(uname).getSmtpTable().
            getRecord(dlg.getSelectedSrvNum());
        dbSmtpTable table = new dbSmtpTable();
        table.append(rec);
        // copy a Mail record from a dialog to the autouser
        dbMailRecord mrec = mContainer.getUser(uname).getMailTable().
            getRecord(0);
        dbMailTable mtable = new dbMailTable();
        mtable.append(mrec);

        //update database
        mUser.setName(uname);
        mUser.setSmtpTable(table);
        mUser.setMailTable(mtable);
        mUser.setSmtpDefaultN(0);

        //update interface labels
        jlManUser.setText(uname);
        jlServer.setText(rec.getIP());

        //update complete verification variable
        mManSel = true;
        mController.getChangesTracker().UsersChanged = true;
      }
      dlg.dispose();
    }
    else {
      mManSel = false;
      BridgeApp.LogError(Logs.Srvcs.SetText(
          "ERROR: NULL pointer to UserContainer!!! (Selection a user)"));
    }
  }

  public void setObject(Object obj) throws HeadlessException {
    userGUIObject mObj = (userGUIObject) obj;
    mUser = (dbUser) mObj.getObject();
    mContainer = mObj.getContainer();

    if (!mInitialized) {
      try {
        Init();
      }
      catch (Exception e) {
        e.printStackTrace();
      }
      mInitialized = true;
    }
    else {
      this.fill();
    }
  }

  public void fill() {
    jlLogin.setText(mUser.getLogin());

    if (mUser.getAddressBook().getSize() > 0) {
      dbMailRecord recipient = mUser.getAddressBook().getRecord(0);
      jAddrField1.setText(recipient.getMail());
      jCommentsField.setText(recipient.getComments());
      jtfTimeout.setText(Integer.toString(recipient.getTimeDelay()));
      jKeyField.setText(recipient.getPubKeyN());
      jMethodDN.setSelectedIndex(recipient.getMethodDN());
      jMethodSign.setSelectedIndex(recipient.getMethodPGPSign());
    }
    else {
      jAddrField1.setText("");
      jCommentsField.setText("");
      jtfTimeout.setText("10");
      jKeyField.setText("");
      jMethodDN.setSelectedIndex(DBridgeClient.Bank().getBox().getMethodDN());
      jMethodSign.setSelectedIndex(DBridgeClient.Bank().getBox().getMethodSign());
    }
    this.jlManUser.setText(mUser.getName());

    if (mUser.getSmtpTable().getSize() > 0) {
      this.jlServer.setText(mUser.getSmtpTable().getRecord(0).getIP());
    }
  }

  public void saveState() {

    mUser.setID(1000); //automatic type TODO: change latter to unique ID
    dbMailRecord recipient = new dbMailRecord();
    recipient.setMail(jAddrField1.getText());
    recipient.setComments(jCommentsField.getText());
    //possible BUG if no number intered
    if (jtfTimeout.getText().compareTo("") > 0) {
      recipient.setTimeDelay(Integer.parseInt(jtfTimeout.getText()));
    }
    recipient.setPubKeyN(jKeyField.getText());
    recipient.setMethodDN(jMethodDN.getSelectedIndex());
    recipient.setMethodPGPSign(jMethodSign.getSelectedIndex());
    dbMailTable table = new dbMailTable();
    table.append(recipient);
    mUser.setAddressBook(table);

    //other parameters are allready saved in
    //     jbSelManUser_actionPerformed
  }

  public boolean check() {
    //Veryfy the address
    if (jAddrField1.getText().length() < 2) {
      JOptionPane.showMessageDialog(null, lbls.getString("inv_address"),
                                    lbls.getString("error"),
                                    1);
      return false;
    }

    if (!this.mManSel) {
      JOptionPane.showMessageDialog(null, lbls.getString("inv_master"),
                                    lbls.getString("error"), 1);
      return false;
    }

    if (Integer.parseInt(this.jtfTimeout.getText()) < 1) {
      JOptionPane.showMessageDialog(null, lbls.getString("inv_time"),
                                    lbls.getString("error"),
                                    1);
      return false;
    }

    if (jKeyField.getText().length() < 1) {
      JOptionPane.showMessageDialog(null, lbls.getString("inv_key"),
                                    lbls.getString("error"), 1);
      return false;
    }

    return true;

  }
}

/*
//========================================================================
// STRING LISTS for SMTP, POP, ADDRESS BOOK lists
//========================================================================
 class dsStringListEditor extends JPanel implements dsGUIInterface{

  private JButton bAdd;
  private JButton bEdit;
  private JButton bRemove;
  private JLabel  lTitle;
  private static String ADD_COMMAND = "list_add";
  private static String REMOVE_COMMAND = "list_remove";
  private static String EDIT_COMMAND = "list_edit";

  private ActionListener mListener = null;

  private JList mainList;
  private dsStringListDBObject mDataObj;
  private boolean mInitialized = false;
  private UserController mController = null;

  private ResourceBundle lbls;

  public dsStringListEditor(UserController controller){
    mInitialized = false;
    mController = controller;
   // mListener = listener;
  }

  public void KeyInputAction(){
  mController.getChangesTracker().UsersChanged=true;
  }


  public void setActionListener(ActionListener listener){
    mListener = listener;
  }

  public Object getSelectedObj(){
    return mainList.getSelectedValue();
  }

  private void Init(){
    Locale currentLocale = BridgeApp.getCurrentLocale();
    lbls = ResourceBundle.getBundle(
        "dbridge.prop.dsUserSettingsBundle",
        currentLocale);


  BorderLayout main_layout = new BorderLayout();
  //main_layout.setHgap(10);
  //main_layout.setVgap(10);
  this.setLayout(main_layout);

  Box mainBox = Box.createVerticalBox();
  lTitle = new JLabel();
  lTitle.setFont(new java.awt.Font("Dialog", 1, 14));
  JPanel titlePan = new JPanel();
  titlePan.setMaximumSize(new Dimension(10000, 40));
  titlePan.setBorder(BorderFactory.createLoweredBevelBorder());
  titlePan.add(lTitle);
  //this.add(titlePan, BorderLayout.NORTH);
   mainBox.add(titlePan);

   JScrollPane mainPane = new JScrollPane();
   mainList = new JList();
   mainList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
   mainPane.getViewport().add(mainList);
   mainPane.setPreferredSize(new Dimension(150, 200));
   //this.add(mainPane, BorderLayout.CENTER);
   mainBox.add(mainPane);

   bAdd = new JButton();
   bAdd.setText(lbls.getString("bAdd"));
   bAdd.setActionCommand(ADD_COMMAND);
   bAdd.addActionListener(mListener);
   bEdit = new JButton();
   bEdit.setText(lbls.getString("bEdit"));
   bEdit.setActionCommand(EDIT_COMMAND);
   bEdit.addActionListener(mListener);
   bRemove = new JButton();
   bRemove.setText(lbls.getString("bRemove"));
   bRemove.setActionCommand(REMOVE_COMMAND);
   bRemove.addActionListener(mListener);

   JPanel butPanel = new JPanel();
   butPanel.setMaximumSize(new Dimension(10000, 40));
   butPanel.setBorder(BorderFactory.createEtchedBorder());
   butPanel.add(bAdd);
   butPanel.add(bEdit);
   butPanel.add(bRemove);

   //this.add(butPanel, BorderLayout.SOUTH);
   mainBox.add(butPanel);
   this.add(mainBox, BorderLayout.CENTER);
   this.add(Box.createVerticalStrut(10), BorderLayout.NORTH);
   this.add(Box.createVerticalStrut(10), BorderLayout.SOUTH);
   this.add(Box.createHorizontalStrut(10), BorderLayout.EAST);
   this.add(Box.createHorizontalStrut(10), BorderLayout.WEST);

  }

  public void setObject(Object data) throws HeadlessException{
    mDataObj = (dsStringListDBObject) data;
    if (!mInitialized) {
      try {
        Init();
        this.mainList.setListData(mDataObj.getVector());
        lTitle.setText(this.mDataObj.toString());
      }
      catch (Exception e) {
        e.printStackTrace();
      }
      mInitialized=true;
    }
    else {
      this.mainList.setListData(mDataObj.getVector());
      lTitle.setText(this.mDataObj.toString());
    }
  }


  public void saveState(){

  }

  public boolean check(){
    return true;
  }

 }
 */
//============================================================================

class dsSMTPPanel
    extends dsServersTablePanel {

  public dsSMTPPanel(UserController controller) {
    super(controller);
  }

  public void setObject(Object data) throws HeadlessException {
    mUser = (dbUser) ( (userGUIObject) data).getObject();
    dbSmtpTable smtp_tab = mUser.getSmtpTable();

    if (!mInitialized) {
      try {
        mainTable = new dsSMTPTable(smtp_tab);
        Init();
        this.mainPane.getViewport().add( (dsSMTPTable) mainTable);
        lTitle.setText(lbls.getString("smtp"));
      }
      catch (Exception e) {
        e.printStackTrace();
      }
      mInitialized = true;
    }
    else {
      ( (dsSMTPTable) mainTable).setData(smtp_tab);
      lTitle.setText(lbls.getString("smtp"));
    }
  }

  public void saveState() {

  }

  public boolean check() {
    return true;
  }

}

//==========================================================================
// POP/IMAP panel
//==========================================================================
class dsPOPPanel
    extends dsServersTablePanel {

  private dbMailTable mDataObj = null;

  /**
   * Constructor for dsMainForm
   * @param controller UserController
   */
  public dsPOPPanel(UserController controller) {
    super(controller);
  }

  /**
   * Constructor for dsUserInfoTree
   * @param table dbMailTable
   */
  public dsPOPPanel(IUpdateUser controller) {
    super(controller);
  }

  public dbMailTable getMailTable() {
    return mDataObj;
  }

  public void setObject(Object data) throws HeadlessException {
    mDataObj = (dbMailTable) data;
    if (!mInitialized) {
      try {
        mainTable = new dsPOPTable(mDataObj);
        Init();
        this.mainPane.getViewport().add( (JTable) mainTable);
        lTitle.setText(lbls.getString("pop"));
      }
      catch (Exception e) {
        e.printStackTrace();
      }
      mInitialized = true;
    }
    else {
      ( (dsPOPTable) mainTable).setData(mDataObj);
      lTitle.setText(lbls.getString("pop"));
    }
  }

  public void saveState() {

  }

  public boolean check() {
    return true;
  }
}

//==========================================================================
// ADDR book panel
//==========================================================================
class dsADDRPanel
    extends dsServersTablePanel {

  public dsADDRPanel(UserController controller) {
    super(controller);
  }

  public dsADDRPanel(IUpdateUser controller) {
    super(controller);
  }

  public void setObject(Object data) throws HeadlessException {
    mUser = (dbUser) data;
    dbMailTable mail_table = mUser.getAddressBook();
    if (!mInitialized) {
      try {
        mainTable = new dsAddrBookTable(mail_table, mUser.getSmtpTable());
        Init();
        this.mainPane.getViewport().add( (JTable) mainTable);
        lTitle.setText(lbls.getString("addr_book"));
        this.disableCheckButton();
      }
      catch (Exception e) {
        e.printStackTrace();
      }
      mInitialized = true;
    }
    else {
      ( (dsAddrBookTable) mainTable).setData(mail_table, mUser.getSmtpTable());
      lTitle.setText(lbls.getString("addr_book"));
    }
  }

  public void saveState() {

  }

  public boolean check() {
    return true;
  }
}

//========================================================================
//      ADMIN SETTINGS
//========================================================================
class dsAdminSettingsPan
    extends JPanel {

  private JPasswordField jtfPass = null;
  private JPasswordField jtfConf = null;
  private JLabel jlAdmin = null;
  private ResourceBundle lbls;
  ChangesTracker mChangesTracker = null;

  public dsAdminSettingsPan(ChangesTracker tracker) {
    mChangesTracker = tracker;
    try {
      Init();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  private void Init() throws Exception {

    //Locale currentLocale = new Locale("en");
    Locale currentLocale = BridgeApp.getCurrentLocale();
    lbls = ResourceBundle.getBundle(
        "dbridge.prop.dsUserSettingsBundle",
        currentLocale);

    Box mainBox = Box.createVerticalBox();
    JPanel pan1 = new JPanel();
    pan1.setMaximumSize(new Dimension(10000, 60));
    pan1.setBorder(BorderFactory.createCompoundBorder(
        BorderFactory.createLoweredBevelBorder(),
        BorderFactory.createEmptyBorder(4, 4, 4, 4)));
    jlAdmin = new JLabel();
    jlAdmin.setFont(new java.awt.Font("Dialog", 1, 14));
    jlAdmin.setText(lbls.getString("adminSet"));
    pan1.add(jlAdmin);
    mainBox.add(pan1);

    JPanel pan2 = new JPanel();
    pan2.setMaximumSize(new Dimension(10000, 60));
    jtfPass = new JPasswordField();
    jtfPass.addKeyListener(new pass_keyAdapter(this));
    jtfConf = new JPasswordField();
    jtfConf.addKeyListener(new pass_keyAdapter(this));

    pan2.setBorder(BorderFactory.createCompoundBorder(
        BorderFactory.createTitledBorder(lbls.getString("chPass")),
        BorderFactory.createEmptyBorder(4, 4, 4, 4)));
    pan2.setLayout(new GridLayout(0, 2, 20, 2));
    pan2.add(new JLabel(lbls.getString("passwd")));
    pan2.add(jtfPass);
    pan2.add(new JLabel(lbls.getString("confirm")));
    pan2.add(jtfConf);
    mainBox.add(pan2);

    //this.add(mainBox);

    mainBox.add(Box.createGlue());
    BorderLayout layout = new BorderLayout();
    this.setLayout(layout);

    this.add(mainBox, BorderLayout.CENTER);
    this.add(Box.createVerticalStrut(10), BorderLayout.NORTH);
    this.add(Box.createVerticalStrut(10), BorderLayout.SOUTH);
    this.add(Box.createHorizontalStrut(10), BorderLayout.EAST);
    this.add(Box.createHorizontalStrut(10), BorderLayout.WEST);

  }

  public boolean check() {
    boolean res = true;
    String pass1 = new String(jtfPass.getPassword());
    if (!pass1.equals(new String(jtfConf.getPassword()))) {
      JOptionPane.showMessageDialog(null,
                                    lbls.getString("invalid_pass"),
                                    lbls.getString("error"),
                                    JOptionPane.ERROR_MESSAGE);
      res = false;
    }
    return res;
  }

  public void passEdit() {
    mChangesTracker.PasswordChanged = true;
  }

  public String getPassword() {
    return new String(jtfPass.getPassword());
  }

}

//=======================================================================
class pass_keyAdapter
    extends java.awt.event.KeyAdapter {
  private dsAdminSettingsPan adaptee;

  pass_keyAdapter(dsAdminSettingsPan obj) {
    adaptee = obj;
  }

  public void keyTyped(KeyEvent e) {
    adaptee.passEdit();
  }

}

//========================================================================


//
//    BACKUP GUI
//
class dsBackupPan
    extends JPanel {

  private JButton jbBackupUsers = null;
  public static String BCKP_USERS_CMD = "bckp_users";
  private JButton jbRestoreUsers = null;
  public static String REST_USERS_CMD = "rest_users";
  private JButton jbBackupServers = null;
  public static String BCKP_SRVRS_CMD = "bckp_srv";
  private JButton jbRestoreServers = null;
  public static String REST_SRVRS_CMD = "rest_srv";

  private ActionListener mListener = null;

  private ResourceBundle lbls;

  public dsBackupPan(ActionListener listener) {
    mListener = listener;
    try {
      Init();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  private void Init() throws Exception {

    //Locale currentLocale = new Locale("en");
    Locale currentLocale = BridgeApp.getCurrentLocale();
    lbls = ResourceBundle.getBundle(
        "dbridge.prop.dsUserSettingsBundle",
        currentLocale);

    Box box = Box.createVerticalBox();

    JPanel pan1 = new JPanel();
    pan1.setMaximumSize(new Dimension(10000, 60));
    pan1.setBorder(BorderFactory.createLoweredBevelBorder());
    JLabel label = new JLabel(lbls.getString("backup"));
    label.setFont(new java.awt.Font("Dialog", 1, 14));
    pan1.add(label);
    box.add(pan1);

    jbBackupUsers = new JButton(lbls.getString("UsersBackup"));
    jbBackupUsers.setIcon(new ImageIcon("./lib/res/datastore.png"));
    jbBackupUsers.setActionCommand(BCKP_USERS_CMD);
    jbBackupUsers.addActionListener(mListener);

    jbRestoreUsers = new JButton(lbls.getString("UsersRestore"));
    jbRestoreUsers.setIcon(new ImageIcon("./lib/res/dataextract.png"));
    jbRestoreUsers.setActionCommand(REST_USERS_CMD);
    jbRestoreUsers.addActionListener(mListener);

    jbBackupServers = new JButton(lbls.getString("ServersBackup"));
    jbBackupServers.setIcon(new ImageIcon("./lib/res/datastore.png"));
    jbBackupServers.setActionCommand(BCKP_SRVRS_CMD);
    jbBackupServers.addActionListener(mListener);

    jbRestoreServers = new JButton(lbls.getString("ServersRestore"));
    jbRestoreServers.setIcon(new ImageIcon("./lib/res/dataextract.png"));
    jbRestoreServers.setActionCommand(REST_SRVRS_CMD);
    jbRestoreServers.addActionListener(mListener);

    Dimension bn_max = new Dimension(200, 27);
    Dimension bn_min = new Dimension(200, 27);
    Dimension bn_pref = new Dimension(200, 27);

    jbBackupUsers.setMinimumSize(bn_min);
    jbRestoreUsers.setMinimumSize(bn_min);
    jbBackupServers.setMinimumSize(bn_min);
    jbRestoreServers.setMinimumSize(bn_min);

    jbBackupUsers.setMaximumSize(bn_max);
    jbRestoreUsers.setMaximumSize(bn_max);
    jbBackupServers.setMaximumSize(bn_max);
    jbRestoreServers.setMaximumSize(bn_max);

    jbBackupUsers.setPreferredSize(bn_pref);
    jbRestoreUsers.setPreferredSize(bn_pref);
    jbBackupServers.setPreferredSize(bn_pref);
    jbRestoreServers.setPreferredSize(bn_pref);

    JPanel pan2 = new JPanel();
    pan2.setLayout(new FlowLayout());
    pan2.setMaximumSize(new Dimension(10000, 60));

    box.add(Box.createVerticalStrut(10));

    Box box1 = Box.createVerticalBox();
    box1.add(jbBackupUsers);
    box1.add(Box.createVerticalStrut(8));
    box1.add(jbRestoreUsers);
    box1.add(Box.createVerticalStrut(8));
    box1.add(jbBackupServers);
    box1.add(Box.createVerticalStrut(8));
    box1.add(jbRestoreServers);
    pan2.add(box1);

    box.add(pan2);

    box.add(Box.createGlue());
    BorderLayout layout = new BorderLayout();
    this.setLayout(layout);

    this.add(box, BorderLayout.CENTER);
    this.add(Box.createVerticalStrut(10), BorderLayout.NORTH);
    this.add(Box.createVerticalStrut(10), BorderLayout.SOUTH);
    this.add(Box.createHorizontalStrut(10), BorderLayout.EAST);
    this.add(Box.createHorizontalStrut(10), BorderLayout.WEST);

  }
}

//=============================================================================
//                   DICOM SERVER GUI
//=============================================================================
class dsDcmServPan
    extends JPanel
    implements dsGUIInterface {

  private JTextField tfIP = null;
  private JTextField tfPort = null;
  private JTextField tfAeT = null;
  private JTextField tfComments = null;

  private dsDcmServerClass mDcmServer = null;
  private boolean mInitialized = false;
  private ServerController mController = null;
  private String mName = null;
  private boolean mEditableFlag = true;
  
  public void setEditableFlag(boolean on){
	  mEditableFlag = on;
  }
  public void setForeground(Color color){
	  if ( tfIP==null ||tfPort==null || tfAeT==null || tfComments==null) return;
	  tfIP.setForeground(color);
	  tfPort.setForeground(color);
	  tfAeT.setForeground(color);
	  tfComments.setForeground(color);
  }
  
  // allow field edit only for servers 1,...,n-1, and dissablse dor server 0
  private void setEditable(boolean on){
	  tfIP.setEditable(on);
	  tfPort.setEditable(on);
	  tfAeT.setEditable(on);
	  tfComments.setEditable(on);
  }

  private void updateName() {
    mName = tfIP.getText() + ":" + tfPort.getText();
    mController.changeDCMName(mName);
  }

  public String getName() {
    return mName;
  }

  private ResourceBundle lbls;

  public dsDcmServPan(ServerController controller) {
    mController = controller;
  }

  private void Init() throws Exception {
    //Locale currentLocale = new Locale("en");
    Locale currentLocale = BridgeApp.getCurrentLocale();
    lbls = ResourceBundle.getBundle(
        "dbridge.prop.dsUserSettingsBundle",
        currentLocale);

    dsDCMGUI_keyAdapter adapter = new dsDCMGUI_keyAdapter(this);

    tfIP = new JTextField();
    tfIP.setMinimumSize(new Dimension(100, 20));
    tfIP.addKeyListener(adapter);
    tfPort = new JTextField();
    tfPort.setMinimumSize(new Dimension(50, 20));
    tfPort.addKeyListener(adapter);
    tfAeT = new JTextField();
    tfAeT.setMinimumSize(new Dimension(50, 20));
    tfAeT.addKeyListener(adapter);
    tfComments = new JTextField();
    tfComments.setMinimumSize(new Dimension(200, 20));
    tfComments.addKeyListener(adapter);
    setEditable(mEditableFlag);
    if ( !mEditableFlag ){
    	setForeground(Color.BLUE);//new Color(255,0,0));
    }

    Box mBox = Box.createVerticalBox();
    JPanel pan1 = new JPanel();
    pan1.setMaximumSize(new Dimension(10000, 60));
    pan1.setBorder(BorderFactory.createLoweredBevelBorder());
    JLabel label1 = new JLabel(lbls.getString("DICOM"));
    label1.setFont(new java.awt.Font("Dialog", 1, 14));
    pan1.add(label1);
    mBox.add(pan1);

    mBox.add(Box.createVerticalStrut(10));

    JPanel pan2 = new JPanel();
    pan2.setMaximumSize(new Dimension(10000, 60));

    pan2.setBorder(BorderFactory.createCompoundBorder(
        BorderFactory.createEtchedBorder(),
        BorderFactory.createEmptyBorder(4, 4, 4, 4)));

    //pan2.setBorder(BorderFactory.createEtchedBorder());
    pan2.setLayout(new GridLayout(0, 2, 20, 2));
    pan2.add(new JLabel(lbls.getString("srvIP")));
    pan2.add(tfIP);
    pan2.add(new JLabel(lbls.getString("port")));
    pan2.add(tfPort);
    pan2.add(new JLabel(lbls.getString("aetitle")));
    pan2.add(tfAeT);
    mBox.add(pan2);
    mBox.add(Box.createVerticalStrut(10));

    JPanel pan3 = new JPanel();
    pan3.setLayout(new GridLayout(1, 1));
    pan3.setMaximumSize(new Dimension(10000, 60));
    pan3.setBorder(BorderFactory.createCompoundBorder(
        BorderFactory.createTitledBorder(lbls.getString("comment")),
        BorderFactory.createEmptyBorder(4, 4, 4, 4)));

    // pan3.setBorder(BorderFactory.createTitledBorder(lbls.getString("comment")));
    pan3.add(tfComments);
    mBox.add(pan3);

//    this.add(mBox);

    mBox.add(Box.createGlue());
    BorderLayout layout = new BorderLayout();
    this.setLayout(layout);

    this.add(mBox, BorderLayout.CENTER);
    this.add(Box.createVerticalStrut(10), BorderLayout.NORTH);
    this.add(Box.createVerticalStrut(10), BorderLayout.SOUTH);
    this.add(Box.createHorizontalStrut(10), BorderLayout.EAST);
    this.add(Box.createHorizontalStrut(10), BorderLayout.WEST);
  }

  public void KeyInputAction() {
    mController.getChangesTracker().ServersChanged = true;
    updateName();
  }

  public void setObject(Object serv) throws HeadlessException {
    mDcmServer = (dsDcmServerClass) serv;
    if (!mInitialized) {
      try {
        Init();
        this.fill();
      }
      catch (Exception e) {
        e.printStackTrace();
      }
      mInitialized = true;

    }
    else {
      this.fill();
    }
  }

  private void fill() {
    tfIP.setText(mDcmServer.getDcmServer());
    tfPort.setText(mDcmServer.getDcmPort());
    tfAeT.setText(mDcmServer.getDcmAeTitle());
    tfComments.setText(mDcmServer.getComments());
  }

  public void saveState() {
    mDcmServer.setDcmServer(tfIP.getText());
    mDcmServer.setDcmPort(tfPort.getText());
    mDcmServer.setDcmAeTitle(tfAeT.getText());
    mDcmServer.setComments(tfComments.getText());
  }

  public boolean check() {
    boolean res = true;
    
    // protect the first record from changes
    //mController.getDcmServers().firstElement().;

    if (tfIP.getText().equals("")) {
      JOptionPane.showMessageDialog(this, lbls.getString("ip_error"));
      res = false;
    }
    if (tfPort.getText().equals("")) {
      JOptionPane.showMessageDialog(this, lbls.getString("port_error"));
      res = false;
    }
    if (tfAeT.getText().equals("")) {
      JOptionPane.showMessageDialog(this, lbls.getString("aet_error"));
      res = false;
    }

    return res;
  }

}

class dsDCMGUI_keyAdapter
    extends java.awt.event.KeyAdapter {
  private dsDcmServPan adaptee;

  dsDCMGUI_keyAdapter(dsDcmServPan obj) {
    adaptee = obj;
  }

  public void keyReleased(KeyEvent e) {
    adaptee.KeyInputAction();
  }

}

//============================================================================
// ADMIN MIME INIT TREE NODE
//============================================================================
class dsFileEditSettingsPan
    extends JPanel {

  private JLabel jlTitle = null;

  // private ResourceBundle lbls;
  private String mTitle = null;
  private String mFileName = null;

  private dsIniFileEditFrame mimeEdit = null;

  public dsFileEditSettingsPan(String title, String filename) {
    mTitle = title;
    mFileName = filename;
    try {
      Init();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  public boolean isChanged() {
    boolean result = false;
    if (mimeEdit != null) {
      result = mimeEdit.isChanged();
    }
    return result;
  }

  public boolean saveFile() {
    boolean result = false;
    if (mimeEdit != null) {
      result = mimeEdit.saveFile();
    }
    return result;

  }

  private void Init() throws Exception {

//  Locale currentLocale = new Locale("en");

//  lbls = ResourceBundle.getBundle(
//      "dbridge.prop.dsMainFormBundle",
//      currentLocale);

    Box mainBox = Box.createVerticalBox();
    JPanel pan1 = new JPanel();
    pan1.setMaximumSize(new Dimension(10000, 60));
    pan1.setBorder(BorderFactory.createCompoundBorder(
        BorderFactory.createLoweredBevelBorder(),
        BorderFactory.createEmptyBorder(4, 4, 4, 4)));
    jlTitle = new JLabel();
    jlTitle.setFont(new java.awt.Font("Dialog", 1, 14));
    jlTitle.setText(mTitle);
    pan1.add(jlTitle);
    mainBox.add(pan1);

    JScrollPane spane = new JScrollPane();
    mimeEdit = new dsIniFileEditFrame(mFileName);
    mimeEdit.setPanelMode();
    spane.getViewport().add(mimeEdit.getContentPane(), null);
    mainBox.add(spane);

    //mainBox.add(Box.createGlue());
    BorderLayout layout = new BorderLayout();
    this.setLayout(layout);

    this.add(mainBox, BorderLayout.CENTER);
    this.add(Box.createVerticalStrut(10), BorderLayout.NORTH);
    this.add(Box.createVerticalStrut(10), BorderLayout.SOUTH);
    this.add(Box.createHorizontalStrut(10), BorderLayout.EAST);
    this.add(Box.createHorizontalStrut(10), BorderLayout.WEST);

  }

}
